package dev.fitko.fitconnect.client.sender;

import dev.fitko.fitconnect.api.domain.model.metadata.data.MimeType;
import dev.fitko.fitconnect.api.domain.model.submission.ServiceType;
import dev.fitko.fitconnect.client.sender.model.SubmissionPayload;
import dev.fitko.fitconnect.client.sender.steps.*;
import lombok.Getter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.net.URI;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import static dev.fitko.fitconnect.api.domain.model.metadata.data.MimeType.APPLICATION_JSON;
import static dev.fitko.fitconnect.api.domain.model.metadata.data.MimeType.APPLICATION_XML;

@Getter
public final class SubmissionBuilder implements BuilderStartStep,
        DataStep,
        DestinationStep,
        ServiceTypeStep,
        BuildStep {

    private static final Logger LOGGER = LoggerFactory.getLogger(SubmissionBuilder.class);

    private UUID destinationId;
    private List<File> attachments = Collections.emptyList();
    private String data;
    private MimeType dataMimeType;
    private ServiceType serviceType;
    private URI schemaUri;

    private SubmissionBuilder() {
    }

    public static BuilderStartStep Builder() {
        return new SubmissionBuilder();
    }

    @Override
    public ServiceTypeStep withDestination(final UUID destinationId) {
        this.destinationId = destinationId;
        return this;
    }

    @Override
    public DataStep withAttachments(final List<File> attachments) {
        this.attachments = attachments == null ? Collections.emptyList() : attachments;
        return this;
    }

    @Override
    public DataStep withAttachment(final File attachment) {
        withAttachments(attachment == null ? Collections.emptyList() : List.of(attachment));
        return this;
    }

    @Override
    public DestinationStep withJsonData(final String jsonData, final URI schemaUri) {
        data = jsonData;
        dataMimeType = APPLICATION_JSON;
        return this;
    }


    @Override
    public DestinationStep withXmlData(final String xmlData, final URI schemaUri) {
        data = xmlData;
        dataMimeType = APPLICATION_XML;
        return this;
    }

    @Override
    public BuildStep withServiceType(final String serviceTypeName, final String leikaKey) {
        serviceType = ServiceType.builder()
                .identifier(leikaKey)
                .name(serviceTypeName)
                .build();
        return this;
    }

    @Override
    public BuildStep withServiceType(final String serviceTypeName, final String description, final String leikaKey) {
        serviceType = ServiceType.builder()
                .identifier(leikaKey)
                .description(description)
                .name(serviceTypeName)
                .build();
        return this;
    }

    @Override
    public SubmissionPayload build() {
        return new SubmissionPayload(this);
    }
}
